# Copyright IBM Corp. 2015 All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


__all__ = [
    'list_opts',
]


import copy

from oslo_config import cfg

from cinder.api import common as api_common
from cinder.api.middleware import auth
from cinder.api.middleware import sizelimit
from cinder.api.views import versions
from cinder.backup import chunkeddriver
from cinder.backup import driver as backup_driver
from cinder.backup.drivers import ceph as ceph_backup
from cinder.backup.drivers import nfs as nfs_backup
from cinder.backup.drivers import swift as swift_backup
from cinder.backup.drivers import tsm as tsm_backup
from cinder.backup import manager as backup_manager
from cinder.cmd import volume as cmd_volume
from cinder.cmd import volume_usage_audit
from cinder.common import config as common_config
from cinder import compute
from cinder.compute import nova
from cinder.db import api as db_api
from cinder.db import base as db_base
from cinder import exception as cinder_exception
from cinder.image import glance
from cinder.image import image_utils
from cinder import keymgr
from cinder.keymgr import conf_key_mgr
from cinder.keymgr import key_mgr
from cinder.openstack.common import eventlet_backdoor
from cinder.openstack.common import periodic_task
from cinder.openstack.common import policy
from cinder import quota
from cinder.scheduler import driver as scheduler_driver
from cinder.scheduler import host_manager as scheduler_host_manager
from cinder.scheduler import manager as scheduler_manager
from cinder.scheduler import scheduler_options
from cinder.scheduler import simple as simple_scheduler
from cinder.scheduler.weights import capacity as capacity_weigher
from cinder.scheduler.weights import volume_number as volume_number_weigher
from cinder import service as cinder_service
from cinder import ssh_utils
from cinder import test
from cinder.transfer import api as transfer_api
from cinder.volume import api as volume_api
from cinder.volume import configuration  # noqa
from cinder.volume import driver as volume_driver
from cinder.volume.drivers import block_device
from cinder.volume.drivers.cloudbyte import options as cloudbyte_options
from cinder.volume.drivers import datera
from cinder.volume.drivers.dell import dell_storagecenter_common
from cinder.volume.drivers.emc import emc_vmax_common
from cinder.volume.drivers.emc import emc_vnx_cli
from cinder.volume.drivers.emc import xtremio
from cinder.volume.drivers import eqlx
from cinder.volume.drivers.fujitsu import eternus_dx_common
from cinder.volume.drivers.fusionio import ioControl
from cinder.volume.drivers import glusterfs
from cinder.volume.drivers.hds import hds
from cinder.volume.drivers.hds import iscsi as hds_iscsi
from cinder.volume.drivers.hds import nfs as hds_nfs
from cinder.volume.drivers.hitachi import hbsd_common
from cinder.volume.drivers.hitachi import hbsd_fc
from cinder.volume.drivers.hitachi import hbsd_horcm
from cinder.volume.drivers.hitachi import hbsd_iscsi
from cinder.volume.drivers.ibm import flashsystem
from cinder.volume.drivers.ibm import gpfs
from cinder.volume.drivers.ibm import ibmnas
from cinder.volume.drivers.ibm import storwize_svc
from cinder.volume.drivers.ibm import xiv_ds8k
from cinder.volume.drivers import lvm
from cinder.volume.drivers.netapp.eseries import iscsi as netapp_iscsi
from cinder.volume.drivers.netapp import options as netapp_options
from cinder.volume.drivers.nexenta import options as nexenta_options
from cinder.volume.drivers import nfs
from cinder.volume.drivers import nimble
from cinder.volume.drivers import openvstorage
from cinder.volume.drivers.prophetstor import options as prophetstor_options
from cinder.volume.drivers import pure
from cinder.volume.drivers import quobyte
from cinder.volume.drivers import rbd
from cinder.volume.drivers import remotefs
from cinder.volume.drivers.san.hp import hp_3par_common
from cinder.volume.drivers.san.hp import hp_lefthand_rest_proxy
from cinder.volume.drivers.san import san
from cinder.volume.drivers import scality
from cinder.volume.drivers import smbfs
from cinder.volume.drivers import solidfire
from cinder.volume.drivers import srb
from cinder.volume.drivers.violin import v6000_common
from cinder.volume.drivers.vmware import vmdk
from cinder.volume.drivers.windows import windows
from cinder.volume.drivers import xio
from cinder.volume.drivers.zfssa import zfssaiscsi
from cinder.volume.drivers.zfssa import zfssanfs
from cinder.volume import manager
from cinder import wsgi
from cinder.zonemanager.drivers.brocade import brcd_fabric_opts
from cinder.zonemanager.drivers.brocade import brcd_fc_zone_driver
from cinder.zonemanager.drivers.cisco import cisco_fabric_opts
from cinder.zonemanager.drivers.cisco import cisco_fc_zone_driver
from cinder.zonemanager import fc_zone_manager

# Option lists that cannot be extracted from other files
# (I.E. lists that are dynamically generated) should be manually
# defined here.

base_san_opts = [
    cfg.StrOpt('fc_fabric_names', default=None,
               help='Comma separated list of fibre channel '
               'fabric names. This list of names is used to'
               ' retrieve other SAN credentials for connecting'
               ' to each SAN fabric'
               ),
    cfg.BoolOpt('cisco_zone_activate',
                default=True,
                help='Indicates whether zone should '
                'be activated or not'),
    cfg.StrOpt('cisco_zone_name_prefix',
               default="openstack",
               help="A prefix to be used when naming zone")]

brcd_san_opts = [
    cfg.BoolOpt('zone_activate',
                default=True,
                help='Indicates whether zone should '
                'be activated or not'),
    cfg.StrOpt('zone_name_prefix',
               default="openstack",
               help="A prefix to be used when naming zone")]


def list_opts():
    """Return a list of oslo.config options available in Cinder.

    The returned list includes all oslo.config options which may be registered
    at runtime by the Cinder.

    Each element of the list is a tuple. The first element is the name of the
    group under which the list of elements in the second element will be
    registered. A group name of None corresponds to the [DEFAULT] group in
    config files.

    This function is also discoverable via the 'cinder' entry point
    under the 'cinder.config.opts' namespace.

    The purpose of this is to allow tools like the Oslo sample config file
    generator to discover the options exposed to users by Cinder.

    All config options must be added to this file in order to become part
    of the cinder.conf.sample file.  For lists, do a copy.deepcopy of the
    list(s) of configuration options.  For individual configuration options
    add a list of one item using [] around the configuration option.

    Hacking check C002 will parse this file and ensure that all files/
    config options lists are included in this file.  In the case that you
    have a file that registers configuration options included from
    another file, import that file above using '# noqa' to indicate that
    the file has been considered for inclusion but does not have
    new configuration options to register.

    :returns: a list of (group_name, opts) tuples
    """
    return[('DEFAULT',
            copy.deepcopy(api_common.api_common_opts) +
            copy.deepcopy(versions.versions_opts) +
            [auth.use_forwarded_for_opt] +
            [sizelimit.max_request_body_size_opt] +
            copy.deepcopy(chunkeddriver.chunkedbackup_service_opts) +
            copy.deepcopy(backup_driver.service_opts) +
            copy.deepcopy(ceph_backup.service_opts) +
            copy.deepcopy(nfs_backup.nfsbackup_service_opts) +
            copy.deepcopy(swift_backup.swiftbackup_service_opts) +
            copy.deepcopy(tsm_backup.tsm_opts) +
            copy.deepcopy(backup_manager.backup_manager_opts) +
            [cmd_volume.host_opt] +
            copy.deepcopy(volume_usage_audit.script_opts) +
            copy.deepcopy(common_config.core_opts) +
            copy.deepcopy(common_config.debug_opts) +
            copy.deepcopy(common_config.global_opts) +
            copy.deepcopy(compute._compute_opts) +
            copy.deepcopy(nova.nova_opts) +
            copy.deepcopy(db_api.db_opts) +
            [db_base.db_driver_opt] +
            copy.deepcopy(cinder_exception.exc_log_opts) +
            copy.deepcopy(glance.glance_opts) +
            copy.deepcopy(glance.glance_core_properties) +
            copy.deepcopy(image_utils.image_helper_opt) +
            copy.deepcopy(conf_key_mgr.key_mgr_opts) +
            copy.deepcopy(keymgr.keymgr_opts) +
            copy.deepcopy(key_mgr.encryption_opts) +
            copy.deepcopy(eventlet_backdoor.eventlet_backdoor_opts) +
            copy.deepcopy(periodic_task.periodic_opts) +
            copy.deepcopy(policy.policy_opts) +
            copy.deepcopy(quota.quota_opts) +
            copy.deepcopy(scheduler_driver.scheduler_driver_opts) +
            copy.deepcopy(scheduler_host_manager.host_manager_opts) +
            [scheduler_manager.scheduler_driver_opt] +
            [scheduler_options.scheduler_json_config_location_opt] +
            copy.deepcopy(simple_scheduler.simple_scheduler_opts) +
            copy.deepcopy(capacity_weigher.capacity_weight_opts) +
            copy.deepcopy(volume_number_weigher.volume_number_weight_opts) +
            copy.deepcopy(cinder_service.service_opts) +
            copy.deepcopy(cinder_service.profiler_opts) +
            copy.deepcopy(ssh_utils.ssh_opts) +
            copy.deepcopy(test.test_opts) +
            copy.deepcopy(transfer_api.volume_transfer_opts) +
            [volume_api.volume_host_opt] +
            [volume_api.volume_same_az_opt] +
            [volume_api.az_cache_time_opt] +
            copy.deepcopy(volume_driver.volume_opts) +
            copy.deepcopy(volume_driver.iser_opts) +
            copy.deepcopy(block_device.volume_opts) +
            copy.deepcopy(cloudbyte_options.cloudbyte_connection_opts) +
            copy.deepcopy(cloudbyte_options.cloudbyte_add_qosgroup_opts) +
            copy.deepcopy(cloudbyte_options.cloudbyte_create_volume_opts) +
            copy.deepcopy(datera.d_opts) +
            copy.deepcopy(dell_storagecenter_common.common_opts) +
            copy.deepcopy(emc_vmax_common.emc_opts) +
            copy.deepcopy(emc_vnx_cli.loc_opts) +
            copy.deepcopy(xtremio.XTREMIO_OPTS) +
            copy.deepcopy(eqlx.eqlx_opts) +
            copy.deepcopy(eternus_dx_common.drv_opts) +
            copy.deepcopy(ioControl.fusionio_iocontrol_opts) +
            copy.deepcopy(glusterfs.volume_opts) +
            copy.deepcopy(hds.HUS_OPTS) +
            copy.deepcopy(hds_iscsi.iSCSI_OPTS) +
            copy.deepcopy(hds_nfs.NFS_OPTS) +
            copy.deepcopy(hbsd_common.volume_opts) +
            copy.deepcopy(hbsd_fc.volume_opts) +
            copy.deepcopy(hbsd_horcm.volume_opts) +
            copy.deepcopy(hbsd_iscsi.volume_opts) +
            copy.deepcopy(flashsystem.flashsystem_opts) +
            copy.deepcopy(gpfs.gpfs_opts) +
            copy.deepcopy(ibmnas.platform_opts) +
            copy.deepcopy(storwize_svc.storwize_svc_opts) +
            copy.deepcopy(xiv_ds8k.xiv_ds8k_opts) +
            copy.deepcopy(lvm.volume_opts) +
            copy.deepcopy(netapp_options.netapp_proxy_opts) +
            copy.deepcopy(netapp_options.netapp_connection_opts) +
            copy.deepcopy(netapp_options.netapp_transport_opts) +
            copy.deepcopy(netapp_options.netapp_basicauth_opts) +
            copy.deepcopy(netapp_options.netapp_cluster_opts) +
            copy.deepcopy(netapp_options.netapp_7mode_opts) +
            copy.deepcopy(netapp_options.netapp_provisioning_opts) +
            copy.deepcopy(netapp_options.netapp_img_cache_opts) +
            copy.deepcopy(netapp_options.netapp_eseries_opts) +
            copy.deepcopy(netapp_options.netapp_nfs_extra_opts) +
            copy.deepcopy(netapp_iscsi.na_opts.netapp_basicauth_opts) +
            copy.deepcopy(netapp_iscsi.na_opts.netapp_connection_opts) +
            copy.deepcopy(netapp_iscsi.na_opts.netapp_eseries_opts) +
            copy.deepcopy(netapp_iscsi.na_opts.netapp_transport_opts) +
            copy.deepcopy(nexenta_options.NEXENTA_CONNECTION_OPTIONS) +
            copy.deepcopy(nexenta_options.NEXENTA_ISCSI_OPTIONS) +
            copy.deepcopy(nexenta_options.NEXENTA_NFS_OPTIONS) +
            copy.deepcopy(nexenta_options.NEXENTA_VOLUME_OPTIONS) +
            copy.deepcopy(nexenta_options.NEXENTA_RRMGR_OPTIONS) +
            copy.deepcopy(nfs.nfs_opts) +
            copy.deepcopy(nimble.nimble_opts) +
            copy.deepcopy(openvstorage.OPTS) +
            copy.deepcopy(prophetstor_options.DPL_OPTS) +
            copy.deepcopy(pure.PURE_OPTS) +
            copy.deepcopy(quobyte.volume_opts) +
            copy.deepcopy(rbd.rbd_opts) +
            copy.deepcopy(remotefs.nas_opts) +
            copy.deepcopy(hp_3par_common.hp3par_opts) +
            copy.deepcopy(hp_lefthand_rest_proxy.hplefthand_opts) +
            copy.deepcopy(san.san_opts) +
            copy.deepcopy(scality.volume_opts) +
            copy.deepcopy(smbfs.volume_opts) +
            copy.deepcopy(solidfire.sf_opts) +
            copy.deepcopy(srb.srb_opts) +
            copy.deepcopy(v6000_common.violin_opts) +
            copy.deepcopy(vmdk.vmdk_opts) +
            copy.deepcopy(windows.windows_opts) +
            copy.deepcopy(xio.XIO_OPTS) +
            copy.deepcopy(zfssaiscsi.ZFSSA_OPTS) +
            copy.deepcopy(zfssanfs.ZFSSA_OPTS) +
            copy.deepcopy(manager.volume_manager_opts) +
            copy.deepcopy(wsgi.socket_opts) +
            copy.deepcopy(wsgi.eventlet_opts) +
            copy.deepcopy(brcd_fabric_opts.brcd_zone_opts) +
            copy.deepcopy(brcd_fc_zone_driver.brcd_opts) +
            copy.deepcopy(brcd_san_opts) +
            copy.deepcopy(cisco_fabric_opts.cisco_zone_opts) +
            copy.deepcopy(cisco_fc_zone_driver.cisco_opts) +
            copy.deepcopy(base_san_opts) +
            copy.deepcopy(fc_zone_manager.zone_manager_opts)
            )]
